<?php declare(strict_types = 1);

namespace Drupal\opencase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\opencase\Utils;

final class TimeBasedFieldUpdater {

    private EntityTypeManagerInterface $entityTypeManager;
    private string $date_field;
    private Utils $utils;
    private string $entity_type;
    private string $date_format;
    private string $bundle;

    final public function __construct(
        EntityTypeManagerInterface $entityTypeManager, 
        Utils $utils,
        string $entity_type, string $bundle, string $date_field, string $date_format = 'Y-m-d'
    )
    {   
        $this->entityTypeManager = $entityTypeManager;
        $this->utils = $utils;
        $this->date_field = $date_field;
        $this->date_format = $date_format;
        $this->entity_type = $entity_type;
        $this->bundle = $bundle;
    }

    final public function update(array $conditions, string $time_elapsed, array $new_values): void {
        $query = $this->entityTypeManager->getStorage($this->entity_type)->getQuery();
        $conditions[] = [$this->date_field, date($this->date_format, strtotime('-'.$time_elapsed)), "<"];
        $conditions[] = ['type', $this->bundle, '='];
        $this->utils->addConditionsToQuery($query, $conditions);
        foreach($query->execute() as $id) {
            $this->updateEntity($id, $new_values);
        }      
    }
    private function updateEntity(int $entity_id, array $new_values): void {
        $entity = $this->entityTypeManager->getStorage($this->entity_type)->load($entity_id);
        foreach($new_values as $new_field=>$new_value) {
            $entity->$new_field = $new_value; 
        }
        $entity->save();
    }
}